#
# The module provides a class to maintain 'trust' in key-objects.
#
# The module is desigened to be used in a DNSSEC registry.
#
#



package DNSSEC::Registry::Trust_Container;
use Net::DNS;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $highly_verbose_history $default_trust_time);

use constant INFINITY => 0xffffffff;  # Time at largest possible vallue. 
use constant ZERO => 0;


require Exporter;
require AutoLoader;

@ISA = qw(Exporter AutoLoader);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT = qw(
);

$VERSION = (qw$Revision: 12 $)[1];
$default_trust_time= 60*60*24*365 ;   # 1 year
$highly_verbose_history=1;


=head1

DNSSEC::Registry::Trust_Container

=head1 SYNOPSIS

  use DNSSEC::Registry::Trust_Container;

=head1 DESCRIPTION

The DNSSEC::Registry::Trust_Container a class to store
Net::DNS::RR::Key objects with information on the state of trust it has.

The intention is that one has easy access to the trust parameters and make
the storage of the information transparent.

The trust parameters are chosen so that several policies can be
implemented. 


=head1 METHODS

=head2 new

    $trust=DNSSEC::Registry::Trust_Container->new($keyrr);
    $trust=DNSSEC::Registry::Trust_Container->new($trustobj_str);

Creates a trust container object containg the $keyrr with default
trust state: PENDING or alternatively creates a the trust container object
from a string representation of the object (as produced by the string method).

=cut

sub new {
    my $class=shift;
    my $enclosed_object=shift;

    if (! ref($enclosed_object)){
	my $self=$class->from_string($enclosed_object) ;
	return $self;

    }

    die "No Net::DNS::RR::Key object as argument\n" unless $enclosed_object->isa('Net::DNS::RR::DNSKEY');

    my $now=time();
    
    my $self={
	t_creation => $now,
	t_start => ZERO,
	t_end=> ZERO,
	history => [],
	enclosed_object => $enclosed_object,
    };
    bless $self, $class;

    #STATE is pending;

    # Add history.
    my @now=gmtime($now);
    my $user="Unknown";
    $user =$ENV{USER} if exists $ENV{USER};
    $user .=  " (". $ENV{LOGNAME} .")" if exists $ENV{LOGNAME};
    my $date=_gmtime2string(@now);
    $self->add_history("object created by " . $user );
    $self->add_history("object creation moment " . $date );
    $self->add_history("object state at creation: ". $self->get_currentstate_string );
    return $self;
}


=head2 is_trusted, is_expired, is_wait, is_always, is_pending, is_revoked,
is_warped.

Returns true if at the time the function is called the trust state is
as incicated as below in the section trust states. Here t_start is the
attribute that sais when the trust starts and t_end determines when
the trust has ended.

=head2 set_always, set_pending, set_revoked, set_warped.

Set the trust state to 'always', 'pending', 'revoked' or 'warped'.

=cut

sub set_always {
    my $self=shift;
    return 1 if $self->is_revoked;
    $self->{'t_start'}=ZERO;
    $self->{'t_end'}=INFINITY;

    my $user="unknown";
    $user =$ENV{USER} if exists $ENV{USER};
    $user .=  " (". $ENV{LOGNAME} .")" if exists $ENV{LOGNAME};
    $self->add_history(_gmtime2string(gmtime()).": state changed to ALWAYS by " . $user );
    return 0;
}


sub is_always {
    my $self=shift;
    return 1 if 
	$self->{'t_start'}==ZERO &&
	    $self->{'t_end'}==INFINITY;
    return 0
}


sub set_pending {
    my $self=shift;
    return 1 if $self->is_revoked;
    $self->{'t_start'}=ZERO;
    $self->{'t_end'}=ZERO;

    my $user="unknown";
    $user =$ENV{USER} if exists $ENV{USER};
    $user .=  " (". $ENV{LOGNAME} .")" if exists $ENV{LOGNAME};
    $self->add_history(_gmtime2string(gmtime()).": state changed to PENDING by " . $user );


    return 0;
}


sub is_pending {
    my $self=shift;
    return 1 if 
	$self->{'t_start'}==ZERO  &&
	    $self->{'t_end'}==ZERO;
    return 0;
}





sub set_revoked {
    my $self=shift;
    $self->{'t_start'}=INFINITY;
    $self->{'t_end'}=INFINITY;
    my $user="unknown";
    $user =$ENV{USER} if exists $ENV{USER};
    $user .=  " (". $ENV{LOGNAME} .")" if exists $ENV{LOGNAME};
    $self->add_history(_gmtime2string(gmtime()).": state changed to REVOKED by " . $user );
}


sub is_revoked {
    my $self=shift;
    return  
	( $self->{'t_start'}==INFINITY &&
	  $self->{'t_end'}==INFINITY )?1:0;
}




sub is_expired {
# Reserved to indicate a special state.
# 
    my $self=shift;
    my $now=time(); 
    return 1 if (
		 $self->{'t_start'} < $self->{'t_end'} &&
		 $self->{'t_end'} != INFINITY &&
		 $self->{'t_start'} != ZERO  &&
		 $self->{'t_end'} < $now
		 );
    return 0;
}




sub is_reserved_1 {
# Reserved to indicate a special state.
# 
    my $self=shift;
    return 1 if (
		 $self->{'t_start'}==
		 $self->{'t_end'} &&
		 $self->{'t_start'} != INFINITY &&
		 $self->{'t_start'} != ZERO
		 );
    return 0;
}


sub is_reserved_2 {
# Reserved to indicate a special state.
# 
    my $self=shift;
    return 1 if (
		 $self->{'t_start'} > $self->{'t_end'} &&
		 $self->{'t_start'} != INFINITY &&
		 $self->{'t_end'} != ZERO
		 );
    
    return 0;
}




sub is_trusted {
    my $self=shift;
    my $now=time();
    return 1 if (
		 $self->get_t_start <= $now &&
		 $self->get_t_end  >= $now 
		 );
    
    return 0;
}


sub set_trusted {
    my $self=shift;
    my $now=time();
    my $trusted_untill=shift;
    $trusted_untill=$now+$default_trust_time unless defined ($trusted_untill);
    $self->set_t_start($now);
    $self->set_t_end($trusted_untill);
    my $user="Unknown";
    $user =$ENV{USER} if exists $ENV{USER};
    $user .=  " (". $ENV{LOGNAME} .")" if exists $ENV{LOGNAME};

    my @now=gmtime($now);
    my $date=_gmtime2string(@now);
    $self->add_history("object set to TRUSTED($now-$trusted_untill) by " . $user . " - " .$date)  ;

}




sub is_pending_revoked {
    my $self=shift;
    my $now=time();
    
    return 1 if (
		 $self->get_t_start >= $now &&
		 $self->get_t_end  <= $now &&
		 $self->get_t_end  !=0 &&         
		 $self->get_t_start != INFINITY
		 );
    
    return 0;
}

sub set_pending_revoked {
    my $self=shift;
    my $tmp_end=$self->get_t_start;
    my $tmp_start=$self->get_t_end;

    $self->{'t_start'}=$tmp_start;
    $self->{'t_end'}=$tmp_end;

}
    
sub set_warped {
    my $self=shift;
    return 1 if $self->is_revoked;
    $self->{'t_start'}=INFINITY;
    $self->{'t_end'}=ZERO;


    my $user="unknown";
    $user =$ENV{USER} if exists $ENV{USER};
    $user .=  " (". $ENV{LOGNAME} .")" if exists $ENV{LOGNAME};
    $self->add_history(_gmtime2string(gmtime()).": state changed to WARPED by " . $user );


    return 0;
}

sub is_warped {
    my $self=shift;
    return 1 if $self->{'t_start'}==INFINITY &&
	$self->{'t_end'}==ZERO;
    return 0;
}



sub get_t_creation {
    my $self=shift;
    return $self->{'t_creation'};
}


sub get_t_start {
    my $self=shift;
    return $self->{'t_start'};
}


sub set_t_start {
    my  $self=shift;
    my $starttime=shift;
    return 0 unless defined $starttime;
    $self->{'t_start'}=$starttime;
    return $self->{'t_start'};
}


sub set_t_end {
    my  $self=shift;
    my $endtime=shift;
    return 0 unless defined $endtime;
    $self->{'t_end'}=$endtime;
    return $self->{'t_end'};
}


sub get_t_end {
    my $self=shift;
    return $self->{'t_end'};
}


sub get_history {
    my $self=shift;
    return $self->{'history'};
}


sub add_history {
    my $self=shift;
    my $history=shift;
    my @history=split "\n",$history;
    push @{$self->{'history'}}, @history;
    return 1;
}


sub get_currentstate_string{
    my $self=shift;
    my $now=time;
    my $start= $self->get_t_start;
    my $end= $self->get_t_end;
    my $state;
    if ( $self->is_revoked ){
	return "REVOKED";
    }elsif ($self->is_warped){
	return "WARPED";
    }elsif ($self->is_pending){
	return "PENDING";
    }elsif ( $now<$start && $start<$end){
	return "WAITING";
    }elsif ( $self->is_reserved_1){
	return "RESERVED_1";
    }elsif ( $self->is_reserved_2){
	return "RESERVED_2";
    }elsif ( $self->is_trusted ){
	return "NOW_TRUSTED";
    }elsif ( $self->is_expired ){
	return "EXPIRED";
    }else{
	return "ERROR: unexpected state";
    }
}


=head1 Trust states measured at 't'

t is the time at which one of the "is_" function is called.

=head2 now

    t_start <= t <= t_end 

the object must be trusted.


=head2 expired		

    t_end < t

the object must not be trusted (trust has expired).


=head2 wait	

    t < t_start

Authenticity is established on the moment the object enters the
database. In most cases that moment will be t_start. t_start in the
'future' thus has special meaning: the object has been authenticated
and may can be trusted but should not be used until t_start.

=head1 Trust states independend of 't'


=head2 always	

    t_start=0, t_end=INFINITE

the object must be trusted always.



=head2 revoked	

    t_start=INFINITE,  t_end=INFINITE

the object must never be trusted again.  trust cannot get into any of
the other states anymore.



=head2 pending	
    
    t_start=0, t_end=0

trust has not been established yet, trust state is pending.  the
object must not be trusted.



=head2 warped	       

    t_end=0  t_start=INFINITE

trust is not pending, nor is the trust revoked.  The object to which
this trust relates has been seen before.  The object must not be
trusted.

=head2 string

    $self->string;

returns a string representation of the object. The format is strictly defined
so that the read method can later create objects from files containing the
output from the string method.


=head2 read

    $self->read($file)

opens and reads $file to create a trust object.

The format of the string and thus the content of the file is as followed:
The file MUST contain one and only one of the following lines:

; TRUST: t_creation : <T_VALUE>

; TRUST: t_start    : <T_VALUE>

and

; TRUST: t_end      : <T_VALUE>

The file MAY contain one or more of the following lines

; TRUST: history    : <STRING>

Other lines that start with ";" are ignored.

The remainder of the file is assumend to be the string representation
(zone file format) of one RR. This string serves as input to
Net::DNS::RR->new. Normally this would be key material

=cut




sub print {
    my $self=shift;
    print $self->string;
    1;
}

sub string {
    my $self=shift;
    my $output="";
    $output.="; TRUST: t_creation : ".$self->get_t_creation . "\n";
    $output.="; TRUST: t_start    : ".$self->get_t_start . "\n";
    $output.="; TRUST: t_end      : ".$self->get_t_end . "\n";
    foreach my $history_line (@{$self->get_history}){
    $output.="; TRUST: history    : ". $history_line ."\n";
   }
    $output.=$self->RR->string ."\n";
    return $output;
}



sub RR {
    my $self=shift;
    return $self->{"enclosed_object"};
}
    


sub from_string { 
    my $class=shift;
    my $input=shift;
    my $t_creation="UNDEF";
    my $t_start="UNDEF";
    my $t_end="UNDEF";
    my @history=();
    my $RRstring;
    my @input=split('\n',$input);
    my $i=0;
    while ($i< @input)
    {   $_=$input[$i];
	$i++;
	if (/^; TRUST: t_creation : (.*)$/ ){
	    die "multiple t_creation, inconsistent file content" if ($t_creation ne "UNDEF");
	    $t_creation=$1;
	}elsif(/^; TRUST: t_start    : (.*)$/){
	    die "multiple t_start, inconsistent file content" if ($t_start ne "UNDEF");
	    $t_start=$1;
	}elsif(/^; TRUST: t_end      : (.*)$/){
	    die "multiple t_end, inconsistent file content" if ($t_end ne "UNDEF");
	    $t_end=$1;
	}elsif(/^; TRUST: history    : (.*)$/){
	    push @history,$1 ;
	}elsif(! /^\s*;/ ){
	    $RRstring .= $_;
	}
	
    }

    #OMK: The following really needs more error checking.
    my $RR=Net::DNS::RR->new($RRstring);
    my $self={
	t_creation => $t_creation,
	t_start => $t_start,
	t_end=> $t_end,
	history => \@history,
	enclosed_object => $RR,
    };
    bless $self, $class;
}





sub read {
    my $class=shift;
    my $filename=shift;

    open(TRUSTFH,"$filename") or die "Cannot open $filename for reading\n";
    

    my $input;
    while (<TRUSTFH>){
	$input.=$_."\n";
    }
    my $self=$class->from_string($input);

    # Add history.
    my $now=time();

    my @now=gmtime($now);
    my $user="Unknown";
    $user =$ENV{USER} if exists $ENV{USER};
    $user .=  " (". $ENV{LOGNAME} .")" if exists $ENV{LOGNAME};
    my $date=_gmtime2string(@now);
    $self->add_history("object read by " . $user . " - " .$date)  if $highly_verbose_history;
    return $self;
}


=head2 identifyer
    
    Returns an unique identifyer of the keymaterial that is enclosed in the
    Trust container. (The DS key digest actually)

=cut

sub identifyer {
    my $self=shift;
    # The container object will need to be a Key RR for this to work

    die "No Net::DNS::RR::DNSKEY object as argument\n" unless $self->RR->isa('Net::DNS::RR::DNSKEY');    
    my $DS=Net::DNS::RR::DS->create($self->RR);
    return $DS->digest;
}
	
	






sub    _gmtime2string {
    my @now=@_;
    my $date=1900+$now[5].
	(($now[4]+1)<10?"0".($now[4]+1):($now[4]+1)).
	    $now[3]."-".$now[2].":".$now[1].".".
		($now[0]<10?"0".$now[0]:$now[0]);
    return $date;
}





=head1 COPYRIGHT

Copyright (c) 2002 RIPE NCC.  Author Olaf M. Kolkman
<net-dns-sec@ripe.net>

All Rights Reserved

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of the author not be used
in advertising or publicity pertaining to distribution of the software
without specific, written prior permission.

THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS; IN NO
EVENT SHALL AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR
CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.


=head1 SEE ALSO

perl(1). Net::DNS, Net::DNS::SEC

=cut








1;
__END__




